@@ -1,4 +1,8 @@
+import sys
+from textwrap import dedent
 import pytest, py
+from _pytest.main import EXIT_NOTESTSCOLLECTED
+
 
 class TestModule:
     def test_failing_import(self, testdir):
@@ -23,6 +27,26 @@ class TestModule:
             "*HINT*",
         ])
 
+    def test_import_prepend_append(self, testdir, monkeypatch):
+        syspath = list(sys.path)
+        monkeypatch.setattr(sys, "path", syspath)
+        root1 = testdir.mkdir("root1")
+        root2 = testdir.mkdir("root2")
+        root1.ensure("x456.py")
+        root2.ensure("x456.py")
+        p = root2.join("test_x456.py")
+        monkeypatch.syspath_prepend(str(root1))
+        p.write(dedent("""\
+            import x456
+            def test():
+                assert x456.__file__.startswith(%r)
+        """ % str(root2)))
+        with root2.as_cwd():
+            reprec = testdir.inline_run("--import-mode=append")
+            reprec.assertoutcome(passed=0, failed=1)
+            reprec = testdir.inline_run()
+            reprec.assertoutcome(passed=1)
+
     def test_syntax_error_in_module(self, testdir):
         modcol = testdir.getmodulecol("this is a syntax error")
         pytest.raises(modcol.CollectError, modcol.collect)
@@ -38,14 +62,10 @@ class TestClass:
             class TestClass1:
                 def __init__(self):
                     pass
-            class TestClass2(object):
-                def __init__(self):
-                    pass
         """)
         result = testdir.runpytest("-rw")
-        result.stdout.fnmatch_lines("""
+        result.stdout.fnmatch_lines_random("""
             WC1*test_class_with_init_warning.py*__init__*
-            *2 warnings*
         """)
 
     def test_class_subclassobject(self, testdir):
@@ -396,9 +416,19 @@ class TestFunction:
                                      ['overridden'])
             def test_overridden_via_param(value):
                 assert value == 'overridden'
+
+            @pytest.mark.parametrize('somevalue', ['overridden'])
+            def test_not_overridden(value, somevalue):
+                assert value == 'value'
+                assert somevalue == 'overridden'
+
+            @pytest.mark.parametrize('other,value', [('foo', 'overridden')])
+            def test_overridden_via_multiparam(other, value):
+                assert other == 'foo'
+                assert value == 'overridden'
         """)
         rec = testdir.inline_run()
-        rec.assertoutcome(passed=1)
+        rec.assertoutcome(passed=3)
 
 
     def test_parametrize_overrides_parametrized_fixture(self, testdir):
@@ -456,6 +486,38 @@ class TestFunction:
         config.pluginmanager.register(MyPlugin2())
         config.hook.pytest_pyfunc_call(pyfuncitem=item)
 
+    def test_multiple_parametrize(self, testdir):
+        modcol = testdir.getmodulecol("""
+            import pytest
+            @pytest.mark.parametrize('x', [0, 1])
+            @pytest.mark.parametrize('y', [2, 3])
+            def test1(x, y):
+                pass
+        """)
+        colitems = modcol.collect()
+        assert colitems[0].name == 'test1[2-0]'
+        assert colitems[1].name == 'test1[2-1]'
+        assert colitems[2].name == 'test1[3-0]'
+        assert colitems[3].name == 'test1[3-1]'
+
+    def test_issue751_multiple_parametrize_with_ids(self, testdir):
+        modcol = testdir.getmodulecol("""
+            import pytest
+            @pytest.mark.parametrize('x', [0], ids=['c'])
+            @pytest.mark.parametrize('y', [0, 1], ids=['a', 'b'])
+            class Test(object):
+                def test1(self, x, y):
+                    pass
+                def test2(self, x, y):
+                    pass
+        """)
+        colitems = modcol.collect()[0].collect()[0].collect()
+        assert colitems[0].name == 'test1[a-c]'
+        assert colitems[1].name == 'test1[b-c]'
+        assert colitems[2].name == 'test2[a-c]'
+        assert colitems[3].name == 'test2[b-c]'
+
+
 class TestSorting:
     def test_check_equality(self, testdir):
         modcol = testdir.getmodulecol("""
@@ -543,7 +605,7 @@ class TestConftestCustomization:
         b = testdir.mkdir("a").mkdir("b")
         b.join("conftest.py").write(py.code.Source("""
             import pytest
-            @pytest.mark.hookwrapper
+            @pytest.hookimpl(hookwrapper=True)
             def pytest_pycollect_makeitem():
                 outcome = yield
                 if outcome.excinfo is None:
@@ -611,9 +673,7 @@ def test_setup_only_available_in_subdir(testdir):
     sub1.join("test_in_sub1.py").write("def test_1(): pass")
     sub2.join("test_in_sub2.py").write("def test_2(): pass")
     result = testdir.runpytest("-v", "-s")
-    result.stdout.fnmatch_lines([
-        "*2 passed*"
-    ])
+    result.assert_outcomes(passed=2)
 
 def test_modulecol_roundtrip(testdir):
     modcol = testdir.getmodulecol("pass", withinit=True)
@@ -850,7 +910,7 @@ def test_unorderable_types(testdir):
     """)
     result = testdir.runpytest()
     assert "TypeError" not in result.stdout.str()
-    assert result.ret == 0
+    assert result.ret == EXIT_NOTESTSCOLLECTED
 
 
 def test_collect_functools_partial(testdir):
