@@ -1,5 +1,7 @@
+import sys
 from _pytest.doctest import DoctestItem, DoctestModule, DoctestTextfile
-import py, pytest
+import py
+import pytest
 
 class TestDoctests:
 
@@ -75,8 +77,6 @@ class TestDoctests:
             assert isinstance(items[0].parent, DoctestModule)
             assert items[0].parent is items[1].parent
 
-    @pytest.mark.xfail('hasattr(sys, "pypy_version_info")', reason=
-                       "pypy leaks one FD")
     def test_simple_doctestfile(self, testdir):
         p = testdir.maketxtfile(test_doc="""
             >>> x = 1
@@ -370,3 +370,126 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-modules",
                                     "--junit-xml=junit.xml")
         reprec.assertoutcome(failed=1)
+
+    def test_doctest_module_session_fixture(self, testdir):
+        """Test that session fixtures are initialized for doctest modules (#768)
+        """
+        # session fixture which changes some global data, which will
+        # be accessed by doctests in a module
+        testdir.makeconftest("""
+            import pytest
+            import sys
+
+            @pytest.yield_fixture(autouse=True, scope='session')
+            def myfixture():
+                assert not hasattr(sys, 'pytest_session_data')
+                sys.pytest_session_data = 1
+                yield
+                del sys.pytest_session_data
+        """)
+        testdir.makepyfile(foo="""
+            import sys
+
+            def foo():
+              '''
+              >>> assert sys.pytest_session_data == 1
+              '''
+
+            def bar():
+              '''
+              >>> assert sys.pytest_session_data == 1
+              '''
+        """)
+        result = testdir.runpytest("--doctest-modules")
+        result.stdout.fnmatch_lines('*2 passed*')
+
+    @pytest.mark.parametrize('config_mode', ['ini', 'comment'])
+    def test_allow_unicode(self, testdir, config_mode):
+        """Test that doctests which output unicode work in all python versions
+        tested by pytest when the ALLOW_UNICODE option is used (either in
+        the ini file or by an inline comment).
+        """
+        if config_mode == 'ini':
+            testdir.makeini('''
+            [pytest]
+            doctest_optionflags = ALLOW_UNICODE
+            ''')
+            comment = ''
+        else:
+            comment = '#doctest: +ALLOW_UNICODE'
+
+        testdir.maketxtfile(test_doc="""
+            >>> b'12'.decode('ascii') {comment}
+            '12'
+        """.format(comment=comment))
+        testdir.makepyfile(foo="""
+            def foo():
+              '''
+              >>> b'12'.decode('ascii') {comment}
+              '12'
+              '''
+        """.format(comment=comment))
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(passed=2)
+
+    def test_unicode_string(self, testdir):
+        """Test that doctests which output unicode fail in Python 2 when
+        the ALLOW_UNICODE option is not used. The same test should pass
+        in Python 3.
+        """
+        testdir.maketxtfile(test_doc="""
+            >>> b'12'.decode('ascii')
+            '12'
+        """)
+        reprec = testdir.inline_run()
+        passed = int(sys.version_info[0] >= 3)
+        reprec.assertoutcome(passed=passed, failed=int(not passed))
+
+
+class TestDocTestSkips:
+    """
+    If all examples in a doctest are skipped due to the SKIP option, then
+    the tests should be SKIPPED rather than PASSED. (#957)
+    """
+
+    @pytest.fixture(params=['text', 'module'])
+    def makedoctest(self, testdir, request):
+        def makeit(doctest):
+            mode = request.param
+            if mode == 'text':
+                testdir.maketxtfile(doctest)
+            else:
+                assert mode == 'module'
+                testdir.makepyfile('"""\n%s"""' % doctest)
+
+        return makeit
+
+    def test_one_skipped(self, testdir, makedoctest):
+        makedoctest("""
+            >>> 1 + 1  # doctest: +SKIP
+            2
+            >>> 2 + 2
+            4
+        """)
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(passed=1)
+
+    def test_one_skipped_failed(self, testdir, makedoctest):
+        makedoctest("""
+            >>> 1 + 1  # doctest: +SKIP
+            2
+            >>> 2 + 2
+            200
+        """)
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(failed=1)
+
+    def test_all_skipped(self, testdir, makedoctest):
+        makedoctest("""
+            >>> 1 + 1  # doctest: +SKIP
+            2
+            >>> 2 + 2  # doctest: +SKIP
+            200
+        """)
+        reprec = testdir.inline_run("--doctest-modules")
+        reprec.assertoutcome(skipped=1)
